use rustracing_jaeger::Span;
use tiny_http::{Request, ResponseBox};

use crate::{
    db::{DBConnection, TfaTypes, UserRole},
    dto,
    routes::{filters::UserInfo, get_reply, AppError}
};

pub fn login(
    span: &Span,
    _: &mut Request,
    db: &mut DBConnection,
    data: dto::requests::Login
) -> Result<ResponseBox, AppError> {
    let user = db
        .find_user(span, &data.username, false)
        .ok_or(AppError::Unauthorized("Invalid username or password"))?;

    if !argon2::verify_encoded(user.password.as_str(), data.password.as_bytes()).unwrap_or(false) {
        return AppError::Unauthorized("Invalid username or password").err();
    }

    if user.role == UserRole::Disabled {
        return AppError::Unauthorized("Account is disabled").err();
    }

    if user.tfa_type != TfaTypes::None {
        if let Some(otp) = data.otp {
            if !super::tfa::verify2fa(&user, otp) {
                return AppError::Unauthorized("Incorrect 2fa").err();
            }
        } else {
            if user.tfa_type == TfaTypes::Email {
                super::tfa::send_2fa_mail(span, &user);
            }

            return get_reply(&dto::responses::Success { statusCode: 200 });
        }
    }

    get_reply(&dto::responses::Login {
        statusCode: 200,
        jwt: super::get_token(span, &user, db)
    })
}

pub fn signup(
    span: &Span,
    _: &mut Request,
    db: &mut DBConnection,
    data: dto::requests::SignUp
) -> Result<ResponseBox, AppError> {
    if db.find_user(span, &data.username, false).is_some() {
        return AppError::BadRequest("Username is already taken").err();
    }

    db.create_user_password(span, data.username, super::hash_password(&data.password));

    get_reply(&dto::responses::Success { statusCode: 200 })
}

pub fn refresh(span: &Span, _: &mut Request, db: &mut DBConnection, info: UserInfo) -> Result<ResponseBox, AppError> {
    db.delete_token(span, info.1.id);

    get_reply(&dto::responses::Login {
        statusCode: 200,
        jwt: super::get_token(span, &info.0, db)
    })
}

pub fn logout_all(
    span: &Span,
    _: &mut Request,
    db: &mut DBConnection,
    info: UserInfo
) -> Result<ResponseBox, AppError> {
    db.delete_all_tokens(span, info.0.id);

    get_reply(&dto::responses::Success { statusCode: 200 })
}

pub fn change_password(
    span: &Span,
    _: &mut Request,
    db: &mut DBConnection,
    mut info: UserInfo,
    data: dto::requests::ChangePassword
) -> Result<ResponseBox, AppError> {
    if !argon2::verify_encoded(info.0.password.as_str(), data.oldPassword.as_bytes()).unwrap_or(false) {
        return AppError::Unauthorized("Old password is wrong").err();
    }

    info.0.password = super::hash_password(&data.newPassword);
    db.save_user(span, &info.0);
    db.delete_all_tokens(span, info.0.id);

    get_reply(&dto::responses::Success { statusCode: 200 })
}
